/************************************************************************************************************
Methods:
1. Add()				Adds entries to LDAP Server
2. Modify()				Modifies entries on LDAP Server with the exception of the distinguished name (DN) attribute
3. ModifyDN()			Modifies the distinguished name attribute for LDAP entries on LDAP server
4. Delete()				Deletes entries from LDAP server
5. Query()				Returns entries information on the basis of some search criteria
6. AttributeAdd()		The method is used to set the value of attributes of the entry, which is to be added in the LDAP Server
7. AttributeModify()	The method is used to set the value of attributes of the entry, which is to be modified in the LDAP Server
8. DN()					Specifies the distinguished name of the entry on LDAP server
9. NewRDN()				Specifies the relative distinguished name of the entry on LDAP server

Properties:
1. Attributes()			It is a semicolon ; separated list of attributes against which the values should be retrieved from the LDAP server in Query method
2. IsError()			Returns 1 if any error occur during the request otherwise 0
3. ErrorReason()		Reports any errors that occur during the request
4. Filter()				It is a filter that defines the conditions that must be fulfilled in order for the search to match a given entry. Its default value is "objectclass = *"
5. FilterFile()			Specifies the name of the file that contains the filter string specification. If filter string is specified in the filter file then the Filter parameter will be ignored
6. MaxRows()			It is the maximum number of entries to be returned as a result of a search
7. Password()			Password corresponds to the user name
8. Port()				If not specified then default port 389 is used for communicating with LDAP server
9. Scope()				Specifies the scope of the search from the entry specified in the DN property
10. ServerName()		Host name or IP address of the LDAP server
11. Sort()				Indicate the attribute to sort query result by
12. SortAscending()		Specify sorting order of the Query results
13. SortCasesensitive()	Specify sorting mechanism of the Query results
14. Timeout()			It is the maximum time (in seconds) allowed for a search
15. UserName()			User name required for accessing the server

************************************************************************************************************/

/***********************************/
/*include all required header files*/
/***********************************/
#include <stdio.h>
#include <conio.h>

/*******************************************************************/
/*import AdvLDAP COM library here to get its methods and properties*/
/*******************************************************************/
#import "C:\AdvLdap.dll" no_namespace

struct DN{
	char AttribName[256];	//Attribute name in the DN
	char Value[256];		//Value against the attribute
	DN *Next;				//pointer containing the next element of the list
};

struct Attribute{
	char AttribName[256];	//Attribute name in an attribute relating to the LDAP entry
	char Value[256];		//Value against the attribute
	long Size;				//size in bytes of the value
	BOOL VaueAdd;			//boolean variable used to specify the info that weather the attribute specified will be overwritten in the previous value of just added
	Attribute *Next;		//pointer containing the next element of the list
};

/*************************/
/*prototype for functions*/
/*************************/
void Add();//Add function
void Delete();//Delete function
void Modify();//Modify function
void ModifyDN();//ModifyDN function
void Query();//Query function

BOOL ColsRows(_variant_t vVal, int &rows, int &cols);//function to get the total rows and columns of the qurey result
void PlaceData(char** value,int row,int col, char data[], int maxCols);//function to place the data in a particular element of the array
void WriteValues(char** values,int rows,int cols);//write the query result in a file

DN* GetDN(DN *Head);//function to get the DN entries
Attribute* GetAttributes(Attribute* Head, BOOL IsModify);//function to get the Attributes entries

/**************/
/*main program*/
/**************/
void main(void)
{
	char Ch;
	do
	{
		printf("\r\nAdd		[A]");
		printf("\r\nDelete		[D]");
		printf("\r\nModify		[M]");
		printf("\r\nModifyDN	[N]");
		printf("\r\nQuery		[Q]");
		printf("\r\nQuit		[X]");
		printf("\r\n\r\nEnter Choice = ");
		scanf("%c", &Ch);
		getchar();
		Ch = toupper(Ch);
		switch (Ch)
		{
			case 'A':Add();break;//call Add function
			case 'D':Delete();break;//call Delete function
			case 'M':Modify();break;//call Modify function
			case 'N':ModifyDN();break;//call ModifyDN function
			case 'Q':Query();break;//call Query function
			case 'X':printf("\r\nGood Bye\r\n");break;
			default: printf("\r\nError: Please enter valid choice\r\n");
		}
	}
	while (Ch != 'X');
}

/******************************************************************/
/*GetDN function: The function is used to get the distinguished name */
/******************************************************************/

DN* GetDN(DN *Head)
{
	DN *Curr,*tmp;
	Curr = tmp = NULL;
	char m_Attrib[256], m_Value[256];// char buffers to get the Attribute name and the corresponding value
	do
	{
		printf("\r\n[Press Enter to quit DN inputs]");
		printf("\r\nEnter RDN Attribute = ");
		gets(m_Attrib);//read Attribute name
		if (strlen(m_Attrib))
		{
			printf("\r\nEnter RDN Attribute's Value = ");
			gets(m_Value);//read value against the attribute

			tmp = new DN;

			strcpy(tmp->AttribName, m_Attrib);
			strcpy(tmp->Value, m_Value);
			tmp->Next = NULL;

			//manage the link list
			if (Curr == NULL)
			{
				Curr = tmp;
				Head = Curr;
			}
			else
			{
				Curr->Next = tmp;
				Curr = tmp;
			}
		}
	}
	while(strlen(m_Attrib) != 0);
	return Head;
}

/********************************************************************/
/*GetAttributes function: The function is used to get the attributes*/
/********************************************************************/
Attribute* GetAttributes(Attribute* Head, BOOL IsModify)
{
	Attribute *Curr,*tmp;
	Curr = tmp = NULL;
	char m_Attrib[256], m_Value[256], m_sSize[10];//char buffer to get the attribute name its value and size. 
												//m_sSize is taken of type char* so as if a used presses Enter on the input then the default value is used.
	long m_Size;//size of the value, default is -1
	char Ch;// to get the modification type flag
	BOOL bCh;// to keep input of Ch in boolean

	do
	{
		printf("\r\n[Press Enter to quit Attributes inputs]");
		printf("\r\nEnter entity Attribute = ");
		gets(m_Attrib);// get attribute name
		if (strlen(m_Attrib))
		{
			printf("\r\nEnter Value = ");
			gets(m_Value);// get attribute's value

			printf("\r\nEnter Size (Data size) = ");
			gets(m_sSize);// size, can be empty string
			m_Size = atoi(m_sSize);
			if (m_Size < 1)
				m_Size = -1;

			if (IsModify)
			{// if function is called by the Modify function
				printf("\r\nOverwrite previous Attribute value [Y/N] = ");
				scanf("%c",&Ch);// get the modification type flag
				getchar();
				Ch = toupper(Ch);
			}

			tmp = new Attribute;

			strcpy(tmp->AttribName, m_Attrib);
			strcpy(tmp->Value, m_Value);
			tmp->Size = m_Size;
			tmp->Next = NULL;

			if (IsModify)
			{				
				if (Ch = 'Y')
					bCh = 0;
				else
					bCh = 1;
			}

			//manage the link list
			if (Curr == NULL)
			{
				Curr = tmp;
				Head = Curr;
			}
			else
			{
				Curr->Next = tmp;
				Curr = tmp;
			}
		}
	}
	while(strlen(m_Attrib) != 0);
return Head;
}

/**************************************************************************/
/*Add function: The function is used to add a new entry in the LDAP Server*/
/**************************************************************************/
void Add()
{
	char m_Server[256], m_Usr[256], m_Pwd[256], m_sPort[5];// chars buffer to get the Server, user, password and port
	int m_Port = -1;

	printf("\r\nEnter Host name of LDAP Server = ");
	gets(m_Server);// get the host name containing the LDAP server
	printf("\r\nEnter Port = ");
	gets(m_sPort);// get port to communicate the data
	m_Port = atoi(m_sPort);
	printf("\r\nEnter User Name accessing the LDAP Server= ");
	gets(m_Usr);// get the username
	printf("\r\nEnter Password corresponding the user = ");
	gets(m_Pwd);// get the password

	DN *dnHead =  NULL;
	dnHead = GetDN(dnHead);//get the DN

	Attribute *AttribsHead = NULL;
	AttribsHead = GetAttributes(AttribsHead, 0);//get the Attributes

	try//for COM library call, try...catch is used to handle exceptions
	{
		ILdapPtr obj;//take smart pointer object obj for COM library
		CoInitialize(NULL);//Initializes the COM library on the current thread
		obj = ILdapPtr("AdvLdap.Ldap");

		obj->PutServerName(m_Server);
		obj->PutUserName(m_Usr);
		obj->PutPassword(m_Pwd);

		if (m_Port > 0)
			obj->PutPort(m_Port);

		DN *TmpDN;
		while(dnHead != NULL)
		{
			obj->DN(dnHead->AttribName,dnHead->Value);
			TmpDN = dnHead;
			dnHead = dnHead->Next;
			delete[] TmpDN;
		}

		Attribute *TmpAttribute;
		while(AttribsHead != NULL)
		{
			obj->AttributeAdd(AttribsHead->AttribName,AttribsHead->Value,AttribsHead->Size);
			TmpAttribute = AttribsHead;
			AttribsHead = AttribsHead->Next;
			delete[] TmpAttribute;
		}

		HRESULT hr = obj->Add();
		if (!SUCCEEDED(hr))//if COM call is not successfull then display error message and return
		{
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
			return;
		}
		//call GetIsError() property of COM library and check its value, if its value is 1, then
		//GetErrorReason() property of COM will contain error message, for which this error occured
		if (obj->GetIsError() == 1)
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
		else
			printf("\r\nSuccessfully Added\r\n");//display successfull message
	}
	catch (_com_error &error)//catch exception here
	{
		printf(TEXT("%S[%X]"), error.ErrorMessage(), error.Error());//display error message
	}
}

/***********************************************************************************************/
/*Modify function: The function is used to modify the attributes of an entry in the LDAP Server*/
/***********************************************************************************************/
void Modify()
{
	char m_Server[256], m_Usr[256], m_Pwd[256], m_sPort[5];// chars buffer to get the Server, user, password and port
	int m_Port = -1;

	printf("\r\nEnter Host name of LDAP Server = ");
	gets(m_Server);// get the host name containing the LDAP server
	printf("\r\nEnter Port = ");
	gets(m_sPort);// get port to communicate the data
	m_Port = atoi(m_sPort);
	printf("\r\nEnter User Name accessing the LDAP Server= ");
	gets(m_Usr);// get the username
	printf("\r\nEnter Password corresponding the user = ");
	gets(m_Pwd);// get the password

	DN *dnHead =  NULL;
	dnHead = GetDN(dnHead);//get dn

	Attribute *AttribsHead = NULL;
	AttribsHead = GetAttributes(AttribsHead, 1);//get attributes

	try//for COM library call, try...catch is used to handle exceptions
	{
		ILdapPtr obj;//take smart pointer object obj for COM library
		CoInitialize(NULL);//Initializes the COM library on the current thread
		obj = ILdapPtr("AdvLdap.Ldap");

		obj->PutServerName(m_Server);
		obj->PutUserName(m_Usr);
		obj->PutPassword(m_Pwd);

		if (m_Port > 0)
			obj->PutPort(m_Port);

		DN *TmpDN;
		while(dnHead != NULL)
		{
			obj->DN(dnHead->AttribName,dnHead->Value);
			TmpDN = dnHead;
			dnHead = dnHead->Next;
			delete[] TmpDN;
		}

		Attribute *TmpAttribute;
		while(AttribsHead != NULL)
		{
			obj->AttributeModify(AttribsHead->AttribName,AttribsHead->Value,AttribsHead->Size,AttribsHead->VaueAdd);
			TmpAttribute = AttribsHead;
			AttribsHead = AttribsHead->Next;
			delete[] TmpAttribute;
		}

		HRESULT hr = obj->Modify();
		if (!SUCCEEDED(hr))//if COM call is not successfull then display error message and return
		{
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
			return;
		}
		//call GetIsError() property of COM library and check its value, if its value is 1, then
		//GetErrorReason() property of COM will contain error message, for which this error occured
		if (obj->GetIsError() == 1)
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
		else
			printf("\r\nSuccessfully Modified\r\n");//display successfull message
	}
	catch (_com_error &error)//catch exception here
	{
		printf(TEXT("%S[%X]"), error.ErrorMessage(), error.Error());//display error message
	}
}

/******************************************************************************************/
/*ModifyDN function: The function is used to modify the RDN of an entry in the LDAP Server*/
/******************************************************************************************/
void ModifyDN()
{
	char m_Server[256], m_Usr[256], m_Pwd[256], m_RDN[256], m_RDNValue[256], m_sPort[5];// chars buffer to get the Server, user, password rdnAttributeName and the rdnAttributeValue and port
	int m_Port = -1;

	printf("\r\nEnter Host name of LDAP Server = ");
	gets(m_Server);// get the host name containing the LDAP server
	printf("\r\nEnter Port = ");
	gets(m_sPort);// get port to communicate the data
	m_Port = atoi(m_sPort);
	printf("\r\nEnter User Name accessing the LDAP Server= ");
	gets(m_Usr);// get the username
	printf("\r\nEnter Password corresponding the user = ");
	gets(m_Pwd);// get the password

	DN *dnHead =  NULL;
	dnHead = GetDN(dnHead);//get DN

	printf("\r\nEnter Attribute of new Relitive Distinguished Name = ");
	gets(m_RDN);// RDN Attribute name
	printf("\r\nEnter Value = ");
	gets(m_RDNValue);// RDN Attribute value

	try//for COM library call, try...catch is used to handle exceptions
	{
		ILdapPtr obj;//take smart pointer object obj for COM library
		CoInitialize(NULL);//Initializes the COM library on the current thread
		obj = ILdapPtr("AdvLdap.Ldap");

		obj->PutServerName(m_Server);
		obj->PutUserName(m_Usr);
		obj->PutPassword(m_Pwd);

		if (m_Port > 0)
			obj->PutPort(m_Port);

		DN *TmpDN;
		while(dnHead != NULL)
		{
			obj->DN(dnHead->AttribName,dnHead->Value);
			TmpDN = dnHead;
			dnHead = dnHead->Next;
			delete[] TmpDN;
		}

		obj->NewRDN(m_RDN, m_RDNValue);

		HRESULT hr = obj->ModifyDN();
		if (!SUCCEEDED(hr))//if COM call is not successfull then display error message and return
		{
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
			return;
		}
		//call GetIsError() property of COM library and check its value, if its value is 1, then
		//GetErrorReason() property of COM will contain error message, for which this error occured
		if (obj->GetIsError() == 1)
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
		else
			printf("\r\nSuccessfully Modified\r\n");//display successfull message
	}
	catch (_com_error &error)//catch exception here
	{
		printf(TEXT("%S[%X]"), error.ErrorMessage(), error.Error());//display error message
	}
}

/*****************************************************************************/
/*Delete function: The function is used to delete an entry in the LDAP Server*/
/*****************************************************************************/
void Delete()
{
	char m_Server[256], m_Usr[256], m_Pwd[256], m_sPort[5];// chars buffer to get the Server, user, password and port
	int m_Port = -1;

	printf("\r\nEnter Host name of LDAP Server = ");
	gets(m_Server);// get the host name containing the LDAP server
	printf("\r\nEnter Port = ");
	gets(m_sPort);// get port to communicate the data
	m_Port = atoi(m_sPort);
	printf("\r\nEnter User Name accessing the LDAP Server= ");
	gets(m_Usr);// get the username
	printf("\r\nEnter Password corresponding the user = ");
	gets(m_Pwd);// get the password

	DN *dnHead = NULL;
	dnHead = GetDN(dnHead);//get DN

	try//for COM library call, try...catch is used to handle exceptions
	{
		ILdapPtr obj;//take smart pointer object obj for COM library
		CoInitialize(NULL);//Initializes the COM library on the current thread
		obj = ILdapPtr("AdvLdap.Ldap");

		obj->PutServerName(m_Server);
		obj->PutUserName(m_Usr);
		obj->PutPassword(m_Pwd);

		if (m_Port > 0)
			obj->PutPort(m_Port);

		DN *TmpDN;
		while(dnHead != NULL)
		{
			obj->DN(dnHead->AttribName,dnHead->Value);
			TmpDN = dnHead;
			dnHead = dnHead->Next;
			delete[] TmpDN;
		}
		HRESULT hr = obj->Delete();
		if (!SUCCEEDED(hr))//if COM call is not successfull then display error message and return
		{
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
			return;
		}
		//call GetIsError() property of COM library and check its value, if its value is 1, then
		//GetErrorReason() property of COM will contain error message, for which this error occured
		if (obj->GetIsError() == 1)
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
		else
			printf("\r\nSuccessfully Deleted\r\n");//display successfull message
	}
	catch (_com_error &error)//catch exception here
	{
		printf(TEXT("%S[%X]"), error.ErrorMessage(), error.Error());//display error message
	}
}

/***************************************************************************/
/*Query function: The function is used to query the data in the LDAP Server*/
/***************************************************************************/
void Query()
{
	// chars buffers to get the Server, user, password, sort info, Search filter, Search filter file, Attributes, scope string, port, max rows and time out
	char m_Server[256], m_Usr[256], m_Pwd[256], m_Sort[256], m_Filter[256], m_FilterFile[256], m_Attributes[256], m_ScopeStr[256], m_sPort[5], m_sRows[5], m_sTimeout[5];
	int m_Port = -1, m_Rows = -1, m_Timeout = -1, m_Scope;//numeric values of port, rows, timeout and scope
	BOOL m_SortAscending = 0, m_SortCase = 0;//boolean variables for accednding/decending caseSensite/caseInsensitive sort flags
	char Ch;

	printf("\r\nEnter Host name of LDAP Server = ");
	gets(m_Server);// get the host name containing the LDAP server
	printf("\r\nEnter Port = ");
	gets(m_sPort);// get port to communicate the data
	m_Port = atoi(m_sPort);
	printf("\r\nEnter User Name accessing the LDAP Server= ");
	gets(m_Usr);// get the username
	printf("\r\nEnter Password corresponding the user = ");
	gets(m_Pwd);// get the password
	printf("\r\nEnter Maximum rows to return in the query = ");
	gets(m_sRows);// get maximum no. of rows that may be returned by the query result
	m_Rows = atoi(m_sRows);
	if (m_Rows == 0)
		m_Rows = -1;
	printf("\r\nEnter Timeout = ");
	gets(m_sTimeout);// get timeout
	m_Timeout = atoi(m_sTimeout);
	if (m_Timeout == 0)
		m_Timeout = -1;
	printf("\r\nEnter attribute to sort query results by = ");
	gets(m_Sort);//// get attribute to sort query results by
	printf("\r\nSort Ascending [Y/N] = ");
	scanf("%c",&Ch);
	getchar();
	Ch = toupper(Ch);
	if(Ch == 'Y')
		m_SortAscending = 1;
	printf("\r\nSort Case Sensitive [Y/N] = ");
	scanf("%c",&Ch);
	getchar();
	if(Ch == 'Y')
		m_SortCase = 1;
	printf("\r\nEnter Search Filter = ");
	gets(m_Filter);
	printf("\r\nEnter Filter file name = ");
	gets(m_FilterFile);
	printf("\r\nBase		[0]");
	printf("\r\nOne Level	[1]");
	printf("\r\nSubtree		[2]");
	printf("\r\nEnter Search scope = ");
	scanf("%d",&m_Scope);
	getchar();
	if (m_Scope <0 || m_Scope >2)
		m_Scope = 1;
	switch (m_Scope){
	case 0:strcpy(m_ScopeStr,"Base");break;
	case 1:strcpy(m_ScopeStr,"OneLevel");break;
	case 2:strcpy(m_ScopeStr,"Subtree");break;
	}

	DN *dnHead =  NULL;
	dnHead = GetDN(dnHead);
	printf("\r\nEnter ';' seperated list of Attributes\r\n against which the values should be retrieved = ");
	gets(m_Attributes);

	try//for COM library call, try...catch is used to handle exceptions
	{
		ILdapPtr obj;//take smart pointer object obj for COM library
		CoInitialize(NULL);//Initializes the COM library on the current thread
		obj = ILdapPtr("AdvLdap.Ldap");

		obj->PutServerName(m_Server);
		obj->PutUserName(m_Usr);
		obj->PutPassword(m_Pwd);

		if (m_Port != -1)
			obj->PutPort(m_Port);

		DN *TmpDN;
		while(dnHead != NULL)
		{
			obj->DN(dnHead->AttribName,dnHead->Value);
			TmpDN = dnHead;
			dnHead = dnHead->Next;
			delete[] TmpDN;
		}

		if (m_Rows != -1)
			obj->PutMaxRows(m_Rows);
		if (m_Timeout != -1)
			obj->PutTimeout(m_Timeout);

		obj->PutSort(m_Sort);
		obj->PutSortAscending(m_SortAscending);
		obj->PutSortCaseSensitive(m_SortCase);
		obj->PutAttributes(m_Attributes);
		obj->PutFilter(m_Filter);
		obj->PutFilterFile(m_FilterFile);
		obj->PutScope(m_ScopeStr);

		_variant_t vVal = obj->Query();
		//call GetIsError() property of COM library and check its value, if its value is 1, then
		//GetErrorReason() property of COM will contain error message, for which this error occured
		if (obj->GetIsError() == 1)
			printf("\r\nError: %s\r\n", (char*)obj->GetErrorReason());//display error message
		else
		{// Got the values in vVal

			printf("\r\nSearch succeeded\r\n");//display successfull message

			int maxRows,maxCols;
			maxRows = maxCols = -1;
			ColsRows(vVal, maxRows, maxCols);// get number of rows and the columns.

			char **values;
			values = new char*[(long)maxRows * maxCols];// one dimentional array of length 'rows*cols'

			IDispatch* p = vVal.pdispVal;
			IQueryPtr queryObj;
			p->QueryInterface(IID_IDispatch,(void**)&queryObj);
			long count = queryObj->GetCount();/*COUNT = COLUMNS*/
			for (int j = 1; j <= count; j++)//loop over each node
			{
				_variant_t retVal = queryObj->GetItem(j);
				IDispatch* pp = retVal.pdispVal;
				IQueryEntriesPtr queryEntriesObj;
				pp->QueryInterface(IID_IDispatch,(void**)&queryEntriesObj);
				
				// place the columns heading in the 0th row
				PlaceData(values,/*row*/0,/*column*/j-1, (char*)queryEntriesObj->GetAttributes(), maxCols);

				_variant_t vals = queryEntriesObj->ValuesList();

				IDispatch* ppp = vals.pdispVal;
				IQuerySubPtr querySubObj;
				ppp->QueryInterface(IID_IDispatch,(void**)&querySubObj);
				
				long count1 = querySubObj->GetCount();/*COUNT1 = ROWS*/
				for (int k=1; k <= count1; k++)
				{
					_variant_t retVal1 = querySubObj->GetItem(k);
					IDispatch* pp1 = retVal1.pdispVal;
					IQueryEntriesSubPtr ptr;
					pp1->QueryInterface(IID_IDispatch,(void**)&ptr);
					_variant_t value = ptr->GetValue();

					long lUpper,lLower;
					SafeArrayGetLBound(value.parray,1,&lLower);
					SafeArrayGetUBound(value.parray,1,&lUpper);
					long dwLeng = lUpper - lLower + 1;

					char netValue[1000]="";
					
					for (int n=0; n<dwLeng; n++)
					{
						VARIANT vrt;
						long kk = n;
						SafeArrayGetElement(value.parray,&kk,&vrt);

						if (vrt.vt == VT_BSTR)
						{
							char ch[256];
							_bstr_t bt;
							bt.Assign(vrt.bstrVal);
							strcpy(ch,(char*)bt);
							strcat(netValue,ch);

							if (n < dwLeng-1)
								strcat(netValue,",");
						}
					}
				// place the data in specifed row and column
				PlaceData(values,k,j-1, netValue, maxCols);
				}
			}

			// write the query result in a file
			
			WriteValues(values,maxRows,maxCols);
		}
	}
	catch (_com_error &error)//catch exception here
	{
		printf(TEXT("%S[%X]"), error.ErrorMessage(), error.Error());//display error message
	}
}

void WriteValues(char** value,int rows,int cols)
{
	LPDWORD tempBytes = new DWORD;
	char* fileName = "c:\\temp\\query.dat";
	HANDLE hFile = CreateFile(fileName,GENERIC_WRITE,NULL,NULL,CREATE_ALWAYS,FILE_FLAG_SEQUENTIAL_SCAN,NULL);
	if (hFile != INVALID_HANDLE_VALUE)
	{
		int element = 0;
		for (int i=0; i<=rows; i++)
			for(int j=0; j<cols; j++)
			{
				element = cols * i + j;
				char ch[1000]="";
				strcpy(ch, value[element]);

				if (j != cols-1)
					sprintf(ch,"%s\t\t",ch);
				else
					sprintf(ch,"%s\r\n",ch);
				WriteFile(hFile,ch,strlen(ch),tempBytes,NULL);
			}
	}
	else
	{
		printf("\r\nOutput file cannot be generated\r\n");
	}
	CloseHandle(hFile);
	printf("The output is placed in the file %s",fileName);
}

void PlaceData(char** valu,int row,int col, char data[], int maxCols)
{
	int leng = strlen(data);
	if (leng == 0)
	{
		leng = 1;
		strcpy(data, "-");
	}
	int element = (int)maxCols*row+col;// get the element in which to place the data
	valu[element] = new char[leng+1];

	data[leng] = '\0';
	strcpy(valu[element],data);
}

BOOL ColsRows(_variant_t vVal, int &rows, int &cols)
{// get the number of rows and columns from the query result.
	IDispatch* p = vVal.pdispVal;
	IQueryPtr queryObj;
	p->QueryInterface(IID_IDispatch,(void**)&queryObj);
	int col = queryObj->GetCount();

	_variant_t retVal = queryObj->GetItem(1);
	IDispatch* pp = retVal.pdispVal;
	IQueryEntriesPtr queryEntriesObj;
	pp->QueryInterface(IID_IDispatch,(void**)&queryEntriesObj);
		
	_variant_t vals = queryEntriesObj->ValuesList();

	IDispatch* ppp = vals.pdispVal;
	IQuerySubPtr querySubObj;
	ppp->QueryInterface(IID_IDispatch,(void**)&querySubObj);
		
	int rws = querySubObj->GetCount();
	rows = rws;
	cols = col;
	return 1;
}
